@@ -112,7 +112,7 @@ unw_print_frmask (char *cp, unsigned int mask)
 static void
 unw_print_abreg (char *cp, unsigned int abreg)
 {
-  static const char *special_reg[16] =
+  static const char * const special_reg[16] =
   {
     "pr", "psp", "@priunat", "rp", "ar.bsp", "ar.bspstore", "ar.rnat",
     "ar.unat", "ar.fpsr", "ar.pfs", "ar.lc",
@@ -303,7 +303,7 @@ typedef bfd_vma unw_word;
 #define UNW_DEC_ABI(fmt, abi, context, arg)			\
   do								\
     {								\
-      static const char *abiname[] =				\
+      static const char * const abiname[] =			\
       {								\
 	"@svr4", "@hpux", "@nt"					\
       };							\
@@ -1044,7 +1044,7 @@ unw_decode_b3_x4 (const unsigned char *dp, unsigned int code, void *arg)
 typedef const unsigned char *(*unw_decoder)
      (const unsigned char *, unsigned int, void *);
 
-static unw_decoder unw_decode_table[2][8] =
+static const unw_decoder unw_decode_table[2][8] =
   {
     /* prologue table: */
     {
