@@ -1,7 +1,7 @@
 /* as.h - global header file
    Copyright 1987, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998,
-   1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
-   Free Software Foundation, Inc.
+   1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
+   2011 Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
 
@@ -38,19 +38,6 @@
 
 #include "alloca-conf.h"
 
-/* Prefer varargs for non-ANSI compiler, since some will barf if the
-   ellipsis definition is used with a no-arguments declaration.  */
-#if defined (HAVE_VARARGS_H) && !defined (__STDC__)
-#undef HAVE_STDARG_H
-#endif
-
-#if defined (HAVE_STDARG_H)
-#define USE_STDARG
-#endif
-#if !defined (USE_STDARG) && defined (HAVE_VARARGS_H)
-#define USE_VARARGS
-#endif
-
 /* Now, tend to the rest of the configuration.  */
 
 /* System include files first...  */
@@ -77,22 +64,7 @@
 #include <errno.h>
 #endif
 
-#ifdef USE_STDARG
 #include <stdarg.h>
-#endif
-
-#ifdef USE_VARARGS
-#include <varargs.h>
-#endif
-
-#if !defined (USE_STDARG) && !defined (USE_VARARGS)
-/* Roll our own.  */
-#define va_alist REST
-#define va_dcl
-typedef int * va_list;
-#define va_start(ARGS)	ARGS = &REST
-#define va_end(ARGS)
-#endif
 
 #include "getopt.h"
 /* The first getopt value for machine-independent long options.
@@ -149,6 +121,10 @@ extern void *realloc ();
 extern char *strstr ();
 #endif
 
+#if !HAVE_DECL_MEMPCPY
+void *mempcpy(void *, const void *, size_t);
+#endif
+
 #if !HAVE_DECL_VSNPRINTF
 extern int vsnprintf(char *, size_t, const char *, va_list);
 #endif
@@ -238,7 +214,7 @@ typedef addressT valueT;
 #define know(p) gas_assert(p)	/* Verify our assumptions!  */
 #endif /* not yet defined */
 #else
-#define know(p)			/* know() checks are no-op.ed  */
+#define know(p)	do {} while (0)	/* know() checks are no-op.ed  */
 #endif
 
 /* input_scrub.c */
@@ -389,6 +365,9 @@ COMMON int flag_strip_local_absolute;
 /* True if we should generate a traditional format object file.  */
 COMMON int flag_traditional_format;
 
+/* TRUE if debug sections should be compressed.  */
+COMMON int flag_compress_debug;
+
 /* TRUE if .note.GNU-stack section with SEC_CODE should be created */
 COMMON int flag_execstack;
 
@@ -429,7 +408,6 @@ enum debug_info_type
   DEBUG_DWARF2
 };
 
-extern enum debug_info_type gen_debug;
 extern enum debug_info_type debug_type;
 extern int use_gnu_debug_info_extensions;
 
@@ -455,7 +433,6 @@ struct _pseudo_type
 
 typedef struct _pseudo_type pseudo_typeS;
 
-#ifdef USE_STDARG
 #if (__GNUC__ >= 2) && !defined(VMS)
 /* for use with -Wformat */
 
@@ -482,13 +459,6 @@ typedef struct _pseudo_type pseudo_typeS;
 
 #endif /* __GNUC__ < 2 || defined(VMS) */
 
-#else /* ! USE_STDARG */
-
-#define PRINTF_LIKE(FCN)	void FCN ()
-#define PRINTF_WHERE_LIKE(FCN)	void FCN ()
-
-#endif /* ! USE_STDARG */
-
 PRINTF_LIKE (as_bad);
 PRINTF_LIKE (as_fatal) ATTRIBUTE_NORETURN;
 PRINTF_LIKE (as_tsktsk);
@@ -605,6 +575,16 @@ COMMON unsigned int  found_comment;
 COMMON char *        found_comment_file;
 #endif
 
+#if defined OBJ_ELF || defined OBJ_MAYBE_ELF
+/* If .size directive failure should be error or warning.  */
+COMMON enum
+  {
+    size_check_error = 0,
+    size_check_warning
+  }
+flag_size_check;
+#endif
+
 #ifndef DOLLAR_AMBIGU
 #define DOLLAR_AMBIGU 0
 #endif
