@@ -1,5 +1,5 @@
 /* tc-xstormy16.c -- Assembler for the Sanyo XSTORMY16.
-   Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
+   Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009, 2010
    Free Software Foundation.
 
    This file is part of GAS, the GNU Assembler.
@@ -122,7 +122,7 @@ md_assemble (char * str)
 
   if (!insn.insn)
     {
-      as_bad (errmsg);
+      as_bad ("%s", errmsg);
       return;
     }
 
@@ -143,7 +143,7 @@ md_operand (expressionS * e)
       SKIP_WHITESPACE ();
       if (*input_line_pointer != '(')
 	{
-	  as_bad ("Expected '('");
+	  as_bad (_("Expected '('"));
 	  goto err;
 	}
       input_line_pointer++;
@@ -152,14 +152,14 @@ md_operand (expressionS * e)
 
       if (*input_line_pointer != ')')
 	{
-	  as_bad ("Missing ')'");
+	  as_bad (_("Missing ')'"));
 	  goto err;
 	}
       input_line_pointer++;
       SKIP_WHITESPACE ();
 
       if (e->X_op != O_symbol)
-	as_bad ("Not a symbolic expression");
+	as_bad (_("Not a symbolic expression"));
       else if (* input_line_pointer == '-')
 	/* We are computing the difference of two function pointers
 	   like this:
@@ -197,7 +197,6 @@ xstormy16_cons_fix_new (fragS *f,
 			expressionS *exp)
 {
   bfd_reloc_code_real_type code;
-  fixS *fix;
 
   if (exp->X_op == O_fptr_symbol)
     {
@@ -223,7 +222,7 @@ xstormy16_cons_fix_new (fragS *f,
  	  break;
 
  	default:
-	  as_bad ("unsupported fptr fixup size %d", nbytes);
+	  as_bad (_("unsupported fptr fixup size %d"), nbytes);
 	  return;
 	}
     }
@@ -235,11 +234,11 @@ xstormy16_cons_fix_new (fragS *f,
     code = BFD_RELOC_32;
   else
     {
-      as_bad ("unsupported fixup size %d", nbytes);
+      as_bad (_("unsupported fixup size %d"), nbytes);
       return;
     }
 
-  fix = fix_new_exp (f, where, nbytes, exp, 0, code);
+  fix_new_exp (f, where, nbytes, exp, 0, code);
 }
 
 /* Called while parsing an instruction to create a fixup.
@@ -266,7 +265,7 @@ xstormy16_cgen_record_fixup_exp (fragS *              frag,
   if (op == O_fptr_symbol)
     {
       if (operand->type != XSTORMY16_OPERAND_IMM16)
-	as_bad ("unsupported fptr fixup");
+	as_bad (_("unsupported fptr fixup"));
       else
 	{
 	  fixP->fx_r_type = BFD_RELOC_XSTORMY16_FPTR16;
