@@ -2,13 +2,13 @@
    format backends.
 
    Copyright 1987, 1990, 1991, 1992, 1993, 1995, 1996, 1997, 1999, 2000,
-   2002, 2003, 2004, 2005 Free Software Foundation, Inc.
+   2002, 2003, 2004, 2005, 2007, 2009, 2010 Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
 
    GAS is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GAS is distributed in the hope that it will be useful,
@@ -71,6 +71,8 @@ struct format_ops {
 
   void (*read_begin_hook) (void);
   void (*symbol_new_hook) (symbolS *);
+  void (*symbol_clone_hook) (symbolS *, symbolS *);
+  void (*adjust_symtab) (void);
 };
 
 extern const struct format_ops elf_format_ops;
