@@ -67,6 +67,7 @@ class Eh_frame_hdr : public Output_section_data
       this->fde_offsets_.push_back(std::make_pair(fde_offset, fde_encoding));
   }
 
+ protected:
   // Set the final data size.
   void
   set_final_data_size();
@@ -75,6 +76,11 @@ class Eh_frame_hdr : public Output_section_data
   void
   do_write(Output_file*);
 
+  // Write to a map file.
+  void
+  do_print_to_mapfile(Mapfile* mapfile) const
+  { mapfile->print_output_data(this, _("** eh_frame_hdr")); }
+
  private:
   // Write the data to the file with the right endianness.
   template<int size, bool big_endian>
@@ -322,6 +328,7 @@ class Eh_frame : public Output_section_data
   unsigned int
   fde_count() const;
 
+ protected:
   // Set the final data size.
   void
   set_final_data_size();
@@ -340,6 +347,11 @@ class Eh_frame : public Output_section_data
   void
   do_write(Output_file*);
 
+  // Write to a map file.
+  void
+  do_print_to_mapfile(Mapfile* mapfile) const
+  { mapfile->print_output_data(this, _("** eh_frame")); }
+
  private:
   // The comparison routine for the CIE map.
   struct Cie_less
@@ -393,7 +405,7 @@ class Eh_frame : public Output_section_data
 	   section_size_type symbol_names_size,
 	   const unsigned char* pcontents,
 	   const unsigned char* pcie,
-	   const unsigned char *pcieend,
+	   const unsigned char* pcieend,
 	   Track_relocs<size, big_endian>* relocs,
 	   Offsets_to_cie* cies,
 	   New_cies* new_cies);
@@ -408,7 +420,7 @@ class Eh_frame : public Output_section_data
 	   const unsigned char* pcontents,
 	   unsigned int offset,
 	   const unsigned char* pfde,
-	   const unsigned char *pfdeend,
+	   const unsigned char* pfdeend,
 	   Track_relocs<size, big_endian>* relocs,
 	   Offsets_to_cie* cies);
 
