@@ -12,4 +12,4 @@ GENERATE_SHLIB_SCRIPT=yes
 GENERATE_PIE_SCRIPT=yes
 NO_SMALL_DATA=yes
 SEPARATE_GOTPLT=12
-SHARABLE_SECTIONS=yes
+IREL_IN_PLT=
