@@ -1,11 +1,11 @@
 SCRIPT_NAME=elf
 OUTPUT_FORMAT="elf32-i386"
+NO_RELA_RELOCS=yes
 TEXT_START_ADDR=0x80000000
-#SHLIB_TEXT_START_ADDR=0x80000000
-NONPAGED_TEXT_START_ADDR=0x80000000
-MAXPAGESIZE=0x1000
+MAXPAGESIZE="CONSTANT (MAXPAGESIZE)"
 ARCH=i386
 MACHINE=
-NOP=0x9090
+NOP=0x90909090
 TEMPLATE_NAME=elf32
 GENERATE_SHLIB_SCRIPT=yes
+NO_SMALL_DATA=yes
