@@ -2,6 +2,5 @@ SCRIPT_NAME=nw
 OUTPUT_FORMAT="elf32-powerpc"
 TEXT_START_ADDR=0x0400000
 DATA_ADDR=0x10000000
-MAXPAGESIZE=0x40000
-NONPAGED_TEXT_START_ADDR=0x0400000
+MAXPAGESIZE="CONSTANT (MAXPAGESIZE)"
 ARCH=powerpc
